<?

namespace App;
use Controllers\BaseController;
use App\Router\Request\HttpRequest;
use App\Router\Middleware\IHttpMiddleware;
use App\Router\Route;
use App\Router\Router;
use App\Router\Session;

// should make the session start

class App {
    private $appName;
    private $version;

    private IHttpMiddleware $middlewarePipeline;

    private Container $container;

    private Router $router;
    private array $controllers = [];

    public function __construct($appName, $version, $diContainer) {
        $this->appName = $appName;
        $this->version = $version;
        $this->$diContainer = $diContainer;
    }
    public function use(IHttpMiddleware $middleware) {
        if ($this->middlewarePipeline === null) {
            $this->middlewarePipeline = $middleware;
        } else {
            // Chain the new middleware to the end of the existing pipeline
            $currentMiddleware = $this->middlewarePipeline;
            while ($currentMiddleware->getNext() !== null) {
                $currentMiddleware = $currentMiddleware->getNext();
            }
            $currentMiddleware->setNext($middleware);
        }
    }

    public function getAppName() {
        return $this->appName;
    }

    public function getVersion() {
        return $this->version;
    }

    public function run(HttpRequest $request) {
        Session::getInstance();
        echo "Running {$this->appName} version {$this->version}\n";
    }

    // public function run(HttpRequest $request) {
    //     if ($this->middlewarePipeline === null) {
    //         // No middleware defined, return the request as-is
    //         return $request;
    //     }

    //     // Exécutez le middleware en utilisant le pipeline
    //     return $this->middlewarePipeline->handle($request, function($request) {
    //         // Logique de gestion principale de la requête ici
    //         echo "Main Request Handling Logic.\n";
    //         return $request;
    //     });
    // }
    

    // should not be hese responsibibilty add in DI
    public function autoAddMiddlewares() {
        $middlewareClasses = $this->findMiddlewareClasses();

        foreach ($middlewareClasses as $middlewareClass) {
            $this->use(new $middlewareClass());
        }
    }

    // Méthode pour rechercher automatiquement les classes de middleware en utilisant la réflexion
    // private function findMiddlewareClasses() {
    //     $middlewareClasses = [];

    //     // Utilisez la réflexion pour obtenir toutes les classes disponibles
    //     $classes = get_declared_classes();

    //     foreach ($classes as $class) {
    //         $reflectionClass = new ReflectionClass($class);
    //         if ($reflectionClass->implementsInterface(MiddlewareInterface::class)) {
    //             // La classe implémente MiddlewareInterface, ajoutez-la à la liste
    //             $middlewareClasses[] = $class;
    //         }
    //     }

    //     return $middlewareClasses;
    // }


    
    public function mapControllers(): void
    {
        $classes = $this->container->getAllRegisteredClassNames();

        foreach ($classes as $class) {
            if ($this->isController($class)) {
                $this->mapControllerRoutes($class);
            }
        }
    }
    private function mapControllerRoutes(string $controllerClass): void
    {
        $reflectionClass = new \ReflectionClass($controllerClass);
        $attributes = $reflectionClass->getAttributes(Route::class);
        $prefix = '';
        if(!empty($attributes)){
            $prefix = $attributes[0]->newInstance()->getPath();
        }
        foreach ($reflectionClass->getMethods(\ReflectionMethod::IS_PUBLIC) as $method) {
            foreach ($method->getAttributes(Route::class) as $attribute) {
                $route = $attribute->newInstance();
                $this->router->add($route->method,$prefix . $route->path, [$controllerClass, $method->getName()]);
            }
        }
    }
    private function isController(string $class): bool
    {
        $reflectionClass = new \ReflectionClass($class);
        return $reflectionClass->isSubclassOf(BaseController::class);
    }
}

?>