<?php
namespace App;
use Controllers\BaseController;
class AppCreator
{
    private Container $container;

    private array $services = [];

    public function __construct()
    {
        $this->container = new Container;
    }

    public function registerService(string $serviceId, callable|string $service): self
    {
        $this->container->set($serviceId, $service);
        $this->services[] = $serviceId;
        return $this;
    }

    public function create()
    {

        if (APP_ENV === 'console') {
            require_once __DIR__ . '/../src/console/Console.php';
            return;
        } elseif (APP_ENV === 'development') {
            $app = new App("HeartTrack", 1, $this->container);
            return $app;
        }


    }

    function AddControllers($namespacePrefix = 'Controllers', $pathToControllers = __DIR__ . 'controller'): self
    {

        $controllerFiles = glob($pathToControllers . '/*.php');

        foreach ($controllerFiles as $file) {
            // Get class name from file name
            $class = basename($file, '.php');
            $fullClassName = $namespacePrefix . '\\' . $class;

            if (!class_exists($fullClassName)) {
                continue;
            }

            // Use reflection to check if class extends BaseController
            $reflectionClass = new \ReflectionClass($fullClassName);
            if ($reflectionClass->isSubclassOf(BaseController::class)) {
                // Register in DI container
                $this->container->set($fullClassName, function () use ($fullClassName) {
                    return new $fullClassName();
                });
            }
        }
        return $this;
    }
}


?>