<?php

class DatabaseQueryExecutor {
    private $connection;
    
    // should be this IDatabaseConnection
    public function __construct(Connection $dbConnection) {
        $this->connection = $dbConnection;
    }

    public function executeQuery(string $query, array $parameters = []): bool {
        $stmt = $this->connection->prepare($query);
        foreach ($parameters as $name => $value) {
            $stmt->bindValue($name, $value[0], $value[1]);
        }
        return $stmt->execute();
    }

    public function executeTransaction(callable $transaction): bool {
        try {
            $this->connection->beginTransaction();
            $transaction($this);
            $this->connection->commit();
            return true;
        } catch (PDOException $e) {
            $this->connection->rollBack();
            throw new Exception('Database transaction failed: ' . $e->getMessage());
        }
    }

    public function fetchAll(string $query, array $parameters = []): array {
        $stmt = $this->connection->prepare($query);
        foreach ($parameters as $name => $value) {
            $stmt->bindValue($name, $value[0], $value[1]);
        }
        $stmt->execute();
        return $stmt->fetchAll(PDO::FETCH_ASSOC);
    }

    public function fetchOne(string $query, array $parameters = []): ?array {
        $stmt = $this->connection->prepare($query);
        foreach ($parameters as $name => $value) {
            $stmt->bindValue($name, $value[0], $value[1]);
        }
        $stmt->execute();
        $result = $stmt->fetch(PDO::FETCH_ASSOC);
        return $result ?: null;
    }
}
?>
