<?php
namespace Database\Mapper;
use Database\Entity;


use Model\User;

class UserMapper {

    /**
     * Convert a User entity to a UserDTO.
     *
     * @param User $user
     * @return UserEntity
     */
    public static function toEntity(User $user): UserEntity {
        return new UserEntity(
            $user->getId(),
            $user->getNom(),
            $user->getPrenom(),
            $user->getEmail(),
            $user->getSexe(),
            $user->getTaille(),
            $user->getPoids(),
            $user->getDateNaissance(),
            $user->getRole()
        );
    }

    /**
     * Convert a UserEntity to a User entity.
     *
     * @param UserEntity $entity
     * @return User
     */
    public static function toModel(UserEntity $entity): User {
        $user = new User(
            $entity->id,
            $entity->nom,
            $entity->prenom,
            $entity->email,
            // Handle password appropriately
            'defaultPassword', // This is a placeholder. Handle with care.
            $entity->sexe,
            $entity->taille,
            $entity->poids,
            $entity->dateNaissance,
            $entity->role
        );

        return $user;
    }
    /**
     * Map SQL result to UserEntity object.
     *
     * @param array $row
     * @return UserEntity
     */
    public static function fromSql(array $row): UserEntity {
        $dateNaissance = new \DateTime($row['date_naissance']); // Adjust the key as per your database column name
        $role = new Role(/* parameters based on your Role class constructor */);

        return new UserEntity(
            $row['id'],
            $row['nom'],
            $row['prenom'],
            $row['email'],
            $row['mot_de_passe'], // Make sure this is handled securely
            $row['sexe'],
            $row['taille'],
            $row['poids'],
            $dateNaissance,
            $role
        );
    }
}
