<?php
namespace Database\Gateway;
use Model\User;
use Repository\IUserRepository;
use DataMapper\UserMapper;
// changer car je suis censé provide une API simplifié genre un QuerryBuilder plus to que sa m'est bon on s'en fou

class UserGateway {
    private DatabaseQueryExecutor $queryExecutor;

    public function __construct(DatabaseQueryExecutor $queryExecutor) {
        $this->queryExecutor = $queryExecutor;
    }

    public function getItemById(int $id): ?UserEntity {
        $result = $this->queryExecutor->fetchOne("SELECT * FROM users WHERE id = :id", ['id' => [$id, PDO::PARAM_INT]]);
        return $result ? UserMapper::fromSql($result) : null;
    }

    public function getItemByEmail(string $email): ?UserEntity {
        $result = $this->queryExecutor->fetchOne("SELECT * FROM users WHERE email = :email", ['email' => [$email, PDO::PARAM_STR]]);
        return $result ? $this->mapper->mapToUser($result) : null;
    }

    public function GetNbItems(): int {
        $result = $this->queryExecutor->fetchOne("SELECT COUNT(*) as count FROM users");
        return $result ? (int)$result['count'] : 0;
    }

    public function GetItems(int $index, int $count, ?string $orderingPropertyName = null, bool $descending = false): array {
        $order = $descending ? 'DESC' : 'ASC';
        $query = "SELECT * FROM users " . ($orderingPropertyName ? "ORDER BY $orderingPropertyName $order " : "") . "LIMIT :index, :count";
        $parameters = ['index' => [$index, PDO::PARAM_INT], 'count' => [$count, PDO::PARAM_INT]];
        $results = $this->queryExecutor->fetchAll($query, $parameters);
        return array_map([$this->mapper, 'mapToUser'], $results);
    }

    public function GetItemsByName(string $substring, int $index, int $count, ?string $orderingPropertyName = null, bool $descending = false): array {
        $order = $descending ? 'DESC' : 'ASC';
        $query = "SELECT * FROM users WHERE CONCAT(first_name, ' ', last_name) LIKE :substring " . ($orderingPropertyName ? "ORDER BY $orderingPropertyName $order " : "") . "LIMIT :index, :count";
        $parameters = ['substring' => ["%$substring%", PDO::PARAM_STR], 'index' => [$index, PDO::PARAM_INT], 'count' => [$count, PDO::PARAM_INT]];
        $results = $this->queryExecutor->fetchAll($query, $parameters);
        return array_map([$this->mapper, 'mapToUser'], $results);
    }

    public function UpdateItem(UserEntity $oldUser, UserEntity $newUser): void {
        // Logique pour mettre à jour un utilisateur
        // Utiliser $this->queryExecutor->executeQuery pour exécuter la requête de mise à jour
    }

    public function AddItem(UserEntity $user): void {
        // Logique pour ajouter un nouvel utilisateur
        // Utiliser $this->queryExecutor->executeQuery pour exécuter la requête d'insertion
    }

    public function DeleteItem(UserEntity $user): bool {
        $success = $this->queryExecutor->executeQuery("DELETE FROM users WHERE id = :id", ['id' => [$user->getId(), PDO::PARAM_INT]]);
        return $success;
    }
}
